"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// packages/@code-like-a-carpenter/foundation-plugin-typescript/src/tables/templates/blind-write.ts
var blind_write_exports = {};
__export(blind_write_exports, {
  blindWriteTemplate: () => blindWriteTemplate,
  makeKeyForBlind: () => makeKeyForBlind
});
module.exports = __toCommonJS(blind_write_exports);
var import_helpers = require("../../helpers.cjs");
var import_computed_fields = require("../computed-fields.cjs");
var import_ensure_table = require("./ensure-table.cjs");
var import_helpers2 = require("./helpers.cjs");
var import_indexes = require("./indexes.cjs");
function blindWriteTemplate(config, model) {
  const {
    fields,
    isPublic: hasPublicId,
    primaryKey,
    table: { tableName },
    ttlConfig,
    typeName
  } = model;
  const key = makeKeyForBlind(config, primaryKey);
  const inputTypeName = `BlindWrite${typeName}Input`;
  const omitInputFields = [
    "createdAt",
    "id",
    "updatedAt",
    "version",
    hasPublicId && "publicId",
    ttlConfig?.fieldName,
    ...fields.filter(({ computeFunction }) => !!computeFunction).map(({ fieldName }) => fieldName)
  ].filter(import_helpers.filterNull).map((f) => `'${f}'`).sort();
  const outputTypeName = `BlindWrite${typeName}Output`;
  return `
export type ${inputTypeName} = Omit<${typeName}, ${omitInputFields.join(
    "|"
  )}> ${ttlConfig ? ` & Partial<Pick<${typeName}, '${ttlConfig.fieldName}'>>` : ""} & Partial<Pick<${typeName}, 'createdAt'>>

export type ${outputTypeName} = ResultType<${typeName}>;
/** */
export async function blindWrite${typeName}(${(0, import_computed_fields.inputName)(
    model
  )}: Readonly<${inputTypeName}>): Promise<Readonly<${outputTypeName}>> {
${(0, import_ensure_table.ensureTableTemplate)(tableName)}
  const now = new Date();
  ${(0, import_computed_fields.defineComputedInputFields)(fields, typeName)}
  const {ExpressionAttributeNames, ExpressionAttributeValues, UpdateExpression} = marshall${typeName}(input, now);

  delete ExpressionAttributeNames['#pk'];
  delete ExpressionAttributeValues[':version'];

  const ean ={
    ...ExpressionAttributeNames,
    '#createdAt': '_ct',
    ${hasPublicId ? "'#publicId': 'publicId'," : ""}
    ${model.secondaryIndexes.filter((index) => (0, import_indexes.indexHasField)("createdAt", model.primaryKey, index)).map((index) => (0, import_indexes.indexToEANPart)(config, index)).flat().join("\n")}
  }
  const eav = {
    ...ExpressionAttributeValues, ':one': 1,
    ':createdAt': now.getTime(),
    ${hasPublicId ? "':publicId': idGenerator()," : ""}
    ${model.secondaryIndexes.filter((index) => (0, import_indexes.indexHasField)("createdAt", model.primaryKey, index)).map((index) => (0, import_indexes.indexToEAVPart)("blind", index)).flat().join("\n")}
  };
  const ue = [
    ...UpdateExpression
      .split(', ')
      .filter((e) => !e.startsWith('#version')),
    '#createdAt = if_not_exists(#createdAt, :createdAt)',
    ${hasPublicId ? "'#publicId = if_not_exists(#publicId, :publicId)'," : ""}
    ${model.secondaryIndexes.filter((index) => (0, import_indexes.indexHasField)("createdAt", model.primaryKey, index)).map(import_indexes.indexToUpdateExpressionPart).flat().join("\n")}
  ].join(', ') + ' ADD #version :one';

  const commandInput: UpdateCommandInput = {
    ExpressionAttributeNames: ean,
    ExpressionAttributeValues: eav,
    Key: ${(0, import_helpers2.objectToString)(key)},
    ReturnConsumedCapacity: 'INDEXES',
    ReturnItemCollectionMetrics: 'SIZE',
    ReturnValues: 'ALL_NEW',
    TableName: tableName,
    UpdateExpression: ue,
  }

  try {
    const {ConsumedCapacity: capacity, ItemCollectionMetrics: metrics, Attributes: item} = await ddbDocClient.send(new UpdateCommand(commandInput));

    assert(capacity, 'Expected ConsumedCapacity to be returned. This is a bug in codegen.');

    assert(item, 'Expected DynamoDB ot return an Attributes prop.');
    assert(item._et === '${typeName}', () => new DataIntegrityError(\`Expected to write ${typeName} but wrote \${item?._et} instead\`));

    return {
      capacity,
      item: unmarshall${typeName}(item),
      metrics,
    }
  }
  catch (err) {
    ${(0, import_helpers2.handleCommonErrors)()}
  }
}
`;
}
function makeKeyForBlind(config, key) {
  if (key.isComposite) {
    const doLegacy = config.legacyEmptySortFieldBehavior && key.sortKeyFields.length === 0;
    return {
      pk: (0, import_helpers2.makeKeyTemplate)(
        key.partitionKeyPrefix,
        key.partitionKeyFields,
        "blind"
      ),
      sk: doLegacy ? `'${key.sortKeyPrefix}#0'` : (0, import_helpers2.makeKeyTemplate)(key.sortKeyPrefix, key.sortKeyFields, "blind")
    };
  }
  return {
    pk: (0, import_helpers2.makeKeyTemplate)(
      key.partitionKeyPrefix,
      key.partitionKeyFields,
      "blind"
    )
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  blindWriteTemplate,
  makeKeyForBlind
});
//# sourceMappingURL=blind-write.cjs.map
