"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// packages/@code-like-a-carpenter/foundation-plugin-typescript/src/tables/templates/helpers.ts
var helpers_exports = {};
__export(helpers_exports, {
  getTypeScriptTypeForField: () => getTypeScriptTypeForField,
  handleCommonErrors: () => handleCommonErrors,
  makeKeyForRead: () => makeKeyForRead,
  makeKeyTemplate: () => makeKeyTemplate,
  marshallField: () => marshallField,
  objectToString: () => objectToString,
  unmarshallField: () => unmarshallField,
  unmarshallFieldValue: () => unmarshallFieldValue
});
module.exports = __toCommonJS(helpers_exports);
var import_assert = __toESM(require("assert"));
var import_helpers = require("../../helpers.cjs");
function getTypeScriptTypeForField({
  fieldName,
  isRequired,
  isScalarType: isScalar,
  typeName
}) {
  if (isRequired) {
    if (isScalar) {
      return [fieldName, `Scalars["${typeName}"]['input']`];
    }
    return [fieldName, typeName];
  }
  if (isScalar) {
    return [`${fieldName}?`, `Maybe<Scalars["${typeName}"]['input']>`];
  }
  return [`${fieldName}?`, `Maybe<${typeName}>`];
}
function marshallField({
  columnName,
  fieldName,
  isDateType
}) {
  if (columnName === "ttl") {
    return `Math.floor(input.${fieldName}.getTime()/1000)`;
  }
  if (isDateType) {
    return `input.${fieldName} === null ? null : input.${fieldName}.toISOString()`;
  }
  return `input.${fieldName}`;
}
function makeKeyTemplate(prefix, fields, mode) {
  const accessors = [
    ...prefix ? [`'${prefix}'`] : [],
    ...fields.map((field) => {
      const { fieldName } = field;
      if (fieldName === "createdAt") {
        if (mode === "blind") {
          return "'createdAt' in input && typeof input.createdAt !== 'undefined' ? input.createdAt.getTime() : now.getTime()";
        }
        if (mode === "create") {
          return "now.getTime()";
        }
        if (mode === "read") {
          return "input.createdAt.getTime()";
        }
        import_assert.default.fail("Invalid mode");
      }
      if (fieldName === "updatedAt") {
        return "now.getTime()";
      }
      if (fieldName === "publicId") {
        if (mode === "create") {
          return "publicId";
        }
      }
      return marshallField(field);
    })
  ].filter(import_helpers.filterNull);
  return `[${accessors.join(", ")}].join('#')`;
}
function objectToString(obj) {
  return `{${Object.entries(obj).map(([k, value]) => `${k}: ${value}`)}}`;
}
function getTransformString(field) {
  if (field.columnName === "ttl") {
    return "(v) => new Date(v * 1000)";
  }
  if (field.isDateType) {
    return "(v) => new Date(v)";
  }
  return "";
}
function unmarshallFieldValue(field) {
  const transformString = getTransformString(field);
  const func = field.isRequired ? "unmarshallRequiredField" : "unmarshallOptionalField";
  const args = [
    "item",
    `'${field.fieldName}'`,
    `[${field.columnNamesForRead.map((c) => `'${c}'`).join(",")}]`,
    transformString
  ];
  return `${func}(${args.join(", ")})`;
}
function unmarshallField(field) {
  const out = unmarshallFieldValue(field);
  return `${field.fieldName}: ${out}`;
}
function handleCommonErrors() {
  return `
    if (err instanceof AssertionError || err instanceof BaseDataLibraryError) {
      throw err;
    }
    if (err instanceof ServiceException) {
      throw new UnexpectedAwsError(err);
    }
    throw new UnexpectedError(err);
  `;
}
function makeKeyForRead(config, key) {
  if (key.isComposite) {
    const doLegacy = config.legacyEmptySortFieldBehavior && key.sortKeyFields.length === 0;
    return {
      pk: makeKeyTemplate(
        key.partitionKeyPrefix,
        key.partitionKeyFields,
        "read"
      ),
      sk: doLegacy ? `'${key.sortKeyPrefix}#0'` : makeKeyTemplate(key.sortKeyPrefix, key.sortKeyFields, "read")
    };
  }
  return {
    pk: makeKeyTemplate(key.partitionKeyPrefix, key.partitionKeyFields, "read")
  };
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  getTypeScriptTypeForField,
  handleCommonErrors,
  makeKeyForRead,
  makeKeyTemplate,
  marshallField,
  objectToString,
  unmarshallField,
  unmarshallFieldValue
});
//# sourceMappingURL=helpers.cjs.map
