// packages/@code-like-a-carpenter/foundation-plugin-typescript/src/tables/computed-fields.ts
import assert from "assert";
function hasComputedFields(model) {
  return model.fields.some((field) => !!field.computeFunction);
}
function inputName(model) {
  return hasComputedFields(model) ? "_input" : "input";
}
function defineComputedInputFields(fields, typeName) {
  const computedFields = fields.filter(({ computeFunction }) => !!computeFunction).map((field) => {
    const { fieldName, computeFunction } = field;
    assert(computeFunction);
    const { importName } = computeFunction;
    return `const ${fieldName}Provider = new ${importName}();

        Object.defineProperty(input, '${fieldName}', {
          enumerable: true,
          /** getter */
          get() {
            return ${fieldName}Provider.compute(this);
          }
        })`;
  }).join("\n");
  if (computedFields) {
    return `
      // This has to be cast because we're adding computed fields on the next
      // lines.
      const input: Marshall${typeName}Input = {..._input} as Marshall${typeName}Input
      ${computedFields}
    `;
  }
  return "";
}
function defineComputedOutputFields(fields) {
  return fields.filter(({ computeFunction }) => !!computeFunction).map((field) => {
    const { fieldName, computeFunction } = field;
    assert(computeFunction);
    const { importName } = computeFunction;
    return `
        const ${fieldName}Provider = new ${importName}();

        Object.defineProperty(result, '${fieldName}', {
          enumerable: true,
          /** getter */
          get() {
            return ${fieldName}Provider.compute(this);
          }
        })
      `;
  }).join("\n");
}
export {
  defineComputedInputFields,
  defineComputedOutputFields,
  hasComputedFields,
  inputName
};
//# sourceMappingURL=computed-fields.mjs.map
